/*
 * Copyright (C) 2012 - 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// MSP430 JTAG DLL debug usage example.
//
// Version 1.1
//

// This file contains an example program that calls the MSP430 JTAG DLL debug functions.

// The MSP430 JTAG DLL functions are used to:
// 1.	Initialize the interface
// 2.	Set the device Vcc
// 3	Open the device
// 4.   Get device information
// 5.1	Erase the device (MAIN FLASH) memory
// 5.2  Verify the device memory was erased succesfully
// 5.3  Program the device memory (example application)
// 5.4	Verify the device memory was programmed succesfully
// 5.5	Program the device memory (reset vector)
// 6.	Reset the device
// 7.1	Read the device program counter
// 7.2	Initialize EEM
// 7.3	Configure breakpoint
// 7.4	Pass breakpoint to the EEM manager
// 7.5	Run device to breakpoint
// 7.6	Read the device state
// 7.7	Clear the device breakpoint
// 7.8	Read the device program counter
// 7.9	Single step the device
// 7.10	Read the device state
// 7.11	Stop the device again
// 7.12	Read the device program counter again
// 8.	Reset the device, and start device execution
// 9.	Close the interface
// 10.1	Determine the error number
// 10.2	Determine the string of the error number

// Notes:
// 1. MSP430_Initialize() must be the first function called
// 2. If called, MSP430_Close() must be the last function called


// #includes. -----------------------------------------------------------------
#include <stdio.h>
#include <time.h> // System timers.
#include "MSP430_Debug.h"
#include "MSP430_EEM.h"

// DLL430 EEM callback
// This function is called in case a EEM specific event occures
void MY_DLL430_CallbackFunc(
					uint32_t MsgId,
					uint32_t wParam,
					int32_t lParam,
					int32_t clientHandle);

// Events set by the EEM callback, to signal the occurence of
// EEM specific events (e.g. breakpoint hit)
// to the user application
HANDLE MyBpEvent		= NULL;

int main()
{
	//DLL API return value
	STATUS_T status;

	//UIF configuration
	CHAR* portNumber;
	LONG version;
	LONG vcc = 3300;
	
	//Device configuration
	union DEVICE_T device;
	// The following MSP430 application is used to demonstrate the debug functionality
	// It is a simple "blinking LED" example (MSP430F6659)

	CHAR data0[] = {
		0xF1, 0x03, 0xB2, 0x40, 0x80, 0x5A, 0x5C, 0x01, 0xD2, 0xD3,
		0x04, 0x02, 0xD2, 0xE3, 0x02, 0x02, 0xB1, 0x40, 0x10, 0x27, 0x00, 0x00, 0x91, 0x83, 0x00, 0x00,
		0x81, 0x93, 0x00, 0x00, 0xF6, 0x27, 0xFA, 0x3F
	};

	// The position where the example application should be placed in flash
	WORD data0_addr;
	// Break at the final "jmp Mainloop" [0xF7, 0x3F]
#define BREAK_ADDR (sizeof(data0)-2)
	// Destination address of "jmp Mainloop"
#define JMP_ADDR 0x16
	// Address of last instruction
#define LAST_ADDR (sizeof(data0)-2)

	// The position where the reset vector should be placed in flash
	WORD data1_addr = 0xfffe;
	LONG state;
	LONG registers[16];
	LONG ssCount;

	//EEM configuration
	const LONG EemClientHandle = 1;
	MessageID_t MsgId;
	BpParameter_t BpBuffer;
	WORD  wBpHandle	= 0;
	DWORD timeout = 10000;

// EEM message id definitions
#define WM_BpMESSAGE      (WM_USER + 1)
#define WM_SsMESSAGE      (WM_USER + 2)
#define WM_StoMESSAGE     (WM_USER + 3)
#define WM_SStateMESSAGE  (WM_USER + 4)
#define WM_WarMESSAGE     (WM_USER + 5)
#define WM_CPUStopMESSAGE (WM_USER + 6)
	
	// Setup for simple test
	// The first FET enumerated will be used
	portNumber = "TIUSB";

	do
	{
		// 1. Initialize the debugger interface
		printf("Initializing the interface: ");
		status = MSP430_Initialize(portNumber, &version);
		printf("MSP430_Initialize(portNumber=%s, version=%d) returns %d\n", portNumber, version, status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 2. Set the device Vcc
		printf("Setting the device Vcc: ");
		status = MSP430_VCC(vcc);
		printf("MSP430_VCC(%d) returns %d\n", vcc, status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 3. Open the device
		// A connection to the device is established
		printf("Opening the device: ");
		status = MSP430_OpenDevice("DEVICE_UNKNOWN", "", 0, 0, DEVICE_UNKNOWN);
		printf("MSP430_OpenDevice() returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 4. Get device information
		// After the call, the passed DEVICE_T union contains all device specific information
		status = MSP430_GetFoundDevice((char*)&device, sizeof(device.buffer));
		printf("MSP430_GetFoundDevice() returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}
		printf(" device.id: %d\n", device.id);
		printf(" device.string: %s\n", device.string);
		printf(" device.mainStart: 0x%04x\n", device.mainStart);
		printf(" device.infoStart: 0x%04x\n", device.infoStart);
		printf(" device.ramEnd: 0x%04x\n", device.ramEnd);
		printf(" device.nBreakpoints: %d\n", device.nBreakpoints);
		printf(" device.emulation: %d\n", device.emulation);
		printf(" device.clockControl: %d\n", device.clockControl);
		printf(" device.lcdStart: 0x%04x\n", device.lcdStart);
		printf(" device.lcdEnd: 0x%04x\n", device.lcdEnd);
		printf(" device.vccMinOp: %d\n", device.vccMinOp);
		printf(" device.vccMaxOp: %d\n", device.vccMaxOp);
		printf(" device.hasTestVpp: %d\n", device.hasTestVpp);

		data0_addr = device.mainStart;

		// The following operations manipulate the device memory using erase, read/write, and verify

		// 5.1 Erase the device memory
		printf("Erasing the device memory: ");
		status = MSP430_Erase(ERASE_MAIN, device.mainStart, 0x10000 - device.mainStart);
		printf("MSP430_Erase(ERASE_MAIN, startAddr=0x%04x, length=0x%04x) returns %d\n",
			   device.mainStart, 0x10000 - device.mainStart, status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 5.2 Verify the device memory was erased succesfully
		printf("Verifying device memory erasure: ");
		status = MSP430_EraseCheck(data0_addr, sizeof(data0));
		printf("MSP430_EraseCheck(startAddr=0x%04x, length=0x%04x) returns %d\n",
			   data0_addr, sizeof(data0), status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 5.3 Program the device memory with data0 (blinking LED)
		printf("Programming the device memory (program): ");
		status = MSP430_Write_Memory(data0_addr, data0, sizeof(data0));
		printf("MSP430_Write_Memory(startAddr=0x%04x, data0, length=0x%04x) returns %d\n",
			   data0_addr, sizeof(data0), status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 5.4 Verify data0 was programmed succesfully
		printf("Verifying device memory programming: ");
		status = MSP430_VerifyMem(data0_addr, sizeof(data0), data0);
		printf("MSP430_VerifyMem(startAddr=0x%04x, length=0x%04x, data0) returns %d\n",
			   data0_addr, sizeof(data0), status);
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 5.5 Program the device memory with data1 (reset vector)
		printf("Programming the device memory (reset vector): ");
		status = MSP430_Write_Memory(data1_addr, (uint8_t*) &data0_addr, sizeof(data0_addr));
		printf("MSP430_Write_Memory(startAddr=0x%04x, data0_addr, length=0x%04x) returns %d\n",
			data1_addr, sizeof(data0_addr), status);
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 6. Reset the device
		// After the reset, the device is halted and stays under debug control
		printf("Resetting the device: ");
		status = MSP430_Reset(RST_RESET, FALSE, FALSE);
		printf("MSP430_Reset(method=RST_RESET, execute=FALSE, releaseJTAG=FALSE) returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}

		// The following operations demonstrate debugging the device.

		// 7.1 Read the device program counter
		// Because the target was halted after reset, the PC should point to start of data0
		printf("Reading the device program counter: ");
		status = MSP430_Read_Registers(registers, MASKREG(PC));
		printf("MSP430_Read_Register(PC=0x%04x) returns %d\n", registers[PC], status);
		if (registers[PC] != data0_addr)
		{
			printf(">>> Error: Incorrect PC value! <<<\n");
		}
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 7.2 Initialize EEM
		// At first, set the message id's for all EEM events
		// The message id's must be provided to MSP430_EEM_Init
		// and match the id's used in the DLL430 EEM callback
		MsgId.uiMsgIdBreakpoint = WM_BpMESSAGE;
		MsgId.uiMsgIdSingleStep = WM_SsMESSAGE;
		MsgId.uiMsgIdStorage    = WM_StoMESSAGE;
		MsgId.uiMsgIdState      = WM_SStateMESSAGE;
		MsgId.uiMsgIdWarning    = WM_WarMESSAGE;
		MsgId.uiMsgIdCPUStopped = WM_CPUStopMESSAGE;

		printf("Initializing EEM: ");
		status = MSP430_EEM_Init(MY_DLL430_CallbackFunc, EemClientHandle, &MsgId);
		printf("MSP430_EEM_Init() returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}

		// 7.3 Configure breakpoint
		// This is a simple code breakpoint, configured to stop
		// the program execution (BP_BRK) when the given address (BREAK_ADDR) is reached
		BpBuffer.bpMode = BP_CODE;
		BpBuffer.bpAction = BP_BRK;
		BpBuffer.lAddrVal = data0_addr + BREAK_ADDR;

		// 7.4 Pass the previously configured breakpoint to the EEM manager
		printf("Setting a device breakpoint: ");
		status = MSP430_EEM_SetBreakpoint(&wBpHandle, &BpBuffer);
		printf("MSP430_EEM_SetBreakpoint(%d, addr=0x%04x) returns %d\n", wBpHandle, data0_addr + BREAK_ADDR, status);
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 7.5 Run device to breakpoint
		// At first, create an event for a breakpoint hit
		// The event will be set within the DLL430 EEM callback
		MyBpEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
		printf("Running the device to the breakpoint: ");
		status = MSP430_Run(RUN_TO_BREAKPOINT, FALSE);
		printf("MSP430_Run(RUN_TO_BREAKPOINT, FALSE) returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}
		printf("Waiting for device breakpoint to be hit: ");
		if (WaitForSingleObject(MyBpEvent, timeout) == WAIT_OBJECT_0)
		{
			ResetEvent(MyBpEvent);
		}
		else
		{
			printf("Timeout occured\n");
			break;
		}

		// 7.6 Read the device state
		// The breakpoint was hit and the target should be stopped at this point
		status = MSP430_State(&state, TRUE, &ssCount);
		printf("MSP430_State(%d, TRUE, %d) returns %d\n", state, ssCount, status);
		if (status != STATUS_OK) 
		{
			break;
		}
		
		// 7.7 Clear the device breakpoint
		// To clear a breakpoint, the breakpoint mode must be changed to BP_CLEAR
		// and passed to the EEM manager
		BpBuffer.bpMode = BP_CLEAR;
		printf("Clearing the device breakpoint: ");
		status = MSP430_EEM_SetBreakpoint(&wBpHandle, &BpBuffer);
		printf("MSP430_EEM_SetBreakpoint(0) returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;		
		}

		// 7.8 Read the device program counter
		// Because the breakpoint was hit and target execution stopped,
		// the PC should point to BREAK_ADDR
		printf("Reading the device program counter: ");
		status = MSP430_Read_Registers(registers, MASKREG(PC));
		printf("MSP430_Read_Register(PC=0x%04x) returns %d\n", registers[PC], status);
		if (registers[PC] != data0_addr + BREAK_ADDR)
		{
			printf(">>> Error: Incorrect PC value! <<<\n");
		}
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 7.9 Single step the device
		// A single step is performed and the target, this call blocks until the
		// single step is completed
		printf("Single stepping the device: ");
		status = MSP430_Run(SINGLE_STEP, FALSE);
		printf("MSP430_Run(SINGLE_STEP, FALSE) returns %d\n", status);
		if (status != STATUS_OK) 
		{
			break;
		}
		
		// 7.10 Read the device state
		// The target should be stopped at this point
		status = MSP430_State(&state, TRUE, &ssCount);
		printf("MSP430_State(%d, TRUE, %d) returns %d\n", state, ssCount, status);
		if (status != STATUS_OK) 
		{
			break;
		}
		
		// 7.11 Read the device program counter
		printf("Reading the device program counter: ");
		status = MSP430_Read_Registers(registers, MASKREG(PC));
		printf("MSP430_Read_Register(PC=0x%04x) returns %d\n", registers[PC], status);
		if (registers[PC] != data0_addr + JMP_ADDR)
		{
			printf(">>> Error: Incorrect PC value! <<<\n");
		}
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 7.10 (Free) Run the device.
		printf("(Free) Running the device: ");
		status = MSP430_Run(FREE_RUN, FALSE);
		printf("MSP430_Run(FREE_RUN, FALSE) returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}
		Sleep(5000);

		// 7.11 Stop the device again
		// This call blocks until the device is stopped
		printf("Stopping the device: ");
		status = MSP430_State(&state, TRUE, FALSE);
		if (status != STATUS_OK)
		{
			break;
		}

		// 7.12 Read the device program counter again
		printf("Reading the device program counter: ");
		status = MSP430_Read_Registers(registers, MASKREG(PC));
		printf("MSP430_Read_Register(PC=0x%04x) returns %d\n", registers[PC], status);
		if ((registers[PC] < data0_addr) || (registers[PC] > data0_addr + LAST_ADDR))
		{
			printf(">>> Error: Incorrect PC value! <<<\n");
		}
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 8. Reset the device, and start device execution
		printf("Resetting the device, and starting device execution: ");
		status = MSP430_Reset(ALL_RESETS, TRUE, FALSE);
		printf("MSP430_Reset(method=ALL_RESETS, execute=TRUE, releaseJTAG=FALSE) returns %d\n", status);
		if (status != STATUS_OK)
		{
			break;
		}
		
		// 9. Close the interface
		printf("Closing the interface: ");
		status = MSP430_Close(FALSE);
		printf("MSP430_Close(FALSE) returns %d\n", status);
		if (status != STATUS_OK) 
		{
			break;
		}
	}
	while (0);

	if (status != STATUS_OK)
	{
		LONG errorNumber;

		// The following operations handle errors.

		printf("Additional error information:\n");
		// 10.1 Determine the error number.
		printf(" Error number: %d\n", errorNumber = MSP430_Error_Number());
		// 10.2 Determine the string of the error number.
		printf(" Error string: %s\n", MSP430_Error_String(errorNumber));
	}

	printf("Press <Enter> to continue");
	getchar();

	return (status == STATUS_OK ? 0 : -1);
}

void MY_DLL430_CallbackFunc(
					uint32_t MsgId,
					uint32_t wParam,
					int32_t lParam,
					int32_t clientHandle)
{
	switch(MsgId)
	{
	case WM_BpMESSAGE:
		printf("   MESSAGE: Breakpoint hit...\n");
		// Set event to signal a breakpoint hit
		SetEvent(MyBpEvent);
		break;
	case WM_SsMESSAGE:
		printf("   MESSAGE: Single step complete...\n");
		break;
	case WM_StoMESSAGE:
		printf("   MESSAGE: Storage on trace buffer...\n");
		break;
	case WM_SStateMESSAGE:
		printf("   MESSAGE: Change in new state of the sequencer...\n");
		break;
	case WM_WarMESSAGE:
		printf("   WARNING: Postmessage warning from DLL...\n");
		break;
	case WM_CPUStopMESSAGE:
		printf("   MESSAGE: Device CPU stopped...\n");
		break;
	default:
		break;
	}
}